/***********************************************************************
*
* Copyright (c) 2021-2024 Barbara Geller and Ansel Sermersheim
*
* This example is free software, released under the BSD 2-Clause license.
* For license details refer to LICENSE provided with this project.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* https://opensource.org/licenses/BSD-2-Clause
*
***********************************************************************/

#include <QtCore>
#include <QtGui>

class MainWindow : public QWidget
{
 public:
   enum ParseArgs {
      Help,
      Version,
      Ok,
   };

   MainWindow();

 private:
   ParseArgs prepareCommandLine(const QStringList &arguments);
   void showHelp();
   void showVersion(QString appName);

   QCommandLineParser m_parser;
};

MainWindow::MainWindow()
{
   QApplication::setApplicationVersion("1.2.6");

   QStringList argList = QCoreApplication::arguments();

   QString appName     = argList.first();
   QString commandLine = argList.join(",  ");

   argList.append("other_A");
   argList.append("other_B");

   ParseArgs parseResult = prepareCommandLine(argList);

   const QStringList args = m_parser.positionalArguments();

   if (parseResult == ParseArgs::Help) {
      showHelp();
      std::exit(0);

   } else if (parseResult == ParseArgs::Version) {
      showVersion(appName);
      std::exit(0);
   }

   //
   setMinimumSize(400, 200);

   QLabel *label_1 = new QLabel();
   label_1->setText("Command Line Arguments and Positional Arguments");
   label_1->setAlignment(Qt::AlignHCenter);

   QLabel *label_2 = new QLabel();
   label_2->setText(commandLine);

   QPushButton *close_pb = new QPushButton();
   close_pb->setText("Close");

   QVBoxLayout *layout1 = new QVBoxLayout();
   layout1->setContentsMargins(20, 20, 20, 20);
   layout1->addWidget(label_1);
   layout1->addSpacing(25);
   layout1->addWidget(label_2);
   layout1->addStretch();

   QHBoxLayout *layout2 = new QHBoxLayout();
   layout2->addStretch();
   layout2->addWidget(close_pb);
   layout2->addStretch();

   QVBoxLayout *layoutMain = new QVBoxLayout(this);
   layoutMain->addLayout(layout1);
   layoutMain->addSpacing(20);
   layoutMain->addLayout(layout2);

   connect(close_pb, &QPushButton::clicked, this, &QWidget::close);
}

MainWindow::ParseArgs MainWindow::prepareCommandLine(const QStringList &arguments)
{
   const QCommandLineOption helpOption = m_parser.addHelpOption();
   const QCommandLineOption versionOption = m_parser.addVersionOption();

   QCommandLineOption formatItem("format", "Description about the format option");
   m_parser.addOption(formatItem);

   QCommandLineOption warningItem("w", "Info about warning option");
   m_parser.addOption(warningItem);

   m_parser.addPositionalArgument("extra_arg_1", "Information about extra_arg_1");
   m_parser.addPositionalArgument("extra_arg_2", "Information about extra_arg_2");

   m_parser.parse(arguments);

   if (m_parser.isSet(helpOption)) {
      return ParseArgs::Help;

   } else if (m_parser.isSet(versionOption)) {
      return ParseArgs::Version;

   }

   return ParseArgs::Ok;
}

void MainWindow::showHelp()
{
   QString text;

   text.append("<html><head/><body><pre>");
   text.append(m_parser.helpText().toHtmlEscaped());
   text.append("</pre></body></html>");

   QMessageBox msgBox(QMessageBox::Information,
      QApplication::applicationName(), text, QMessageBox::Ok);

   msgBox.exec();
}

void MainWindow::showVersion(QString appName)
{
   QString text;

   text.append("<html><body><pre>");
   text.append(appName + "<br>");
   text.append("Version # " + QApplication::applicationVersion() + "<br>");
   text.append("</pre></body></html>");

   QMessageBox msgBox(QMessageBox::Information,
      QApplication::applicationName(), text, QMessageBox::Ok);

   msgBox.exec();
}

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);

   MainWindow *window = new MainWindow();
   window->show();

   return app.exec();
}
